/*
 * definitions for Fabric Management Agent
 */

#include <stdint.h>
#include <errno.h>

#define FMA_MAX_ERROR_DEPTH 10
#define FMA_MAX_ERROR_LEN 256
#define FMA_ERROR(S) do {						\
  fma_save_error(__FILE__, __LINE__, errno, (S));			\
  goto except;								\
} while (0)

#define LF_ERROR FMA_ERROR

enum fma_xbar_types {
  FMA_XT_ANY,		/* no constraints on xbar types */
  FMA_XT_ID_ONLY,	/* fabric has only xbars with IDs */
  FMA_XT_NO_IDS		/* fabric has no xbars with IDs */
};

/*
 * structure for all global data 
 */
struct fma_vars {

  uint32_t done;	/* set to 1 when time to exit */

  /* settings */
  int debug;
  enum fma_xbar_types xbar_types;
  
  char *fma_run;	/* base dir for fma stuff */
  char hostname[LF_STRING_LEN];	/* local hostname */
  struct lf_host *my_host;	/* host struct in map */

  struct fma_myri *myri;	/* Myricom network info */
  int myri_ready;		/* TRUE once Myrinet ready to go */

  struct fma_fms *fms;		/* FMS related vars */

  struct fma_settings *settings;	/* settable paramaters */

  struct lf_fabric *fabric;	/* current fabric */

  int standalone;
  struct fma_standalone_data *stand;	/* standalone info */

  struct fma_tunnel *tunnel;	/* message tunnel data */

  /* Error handling messages */
  int error_index;
  char error_str[FMA_MAX_ERROR_DEPTH][FMA_MAX_ERROR_LEN+1];
  char *error_file[FMA_MAX_ERROR_DEPTH];
  int error_line[FMA_MAX_ERROR_DEPTH];
  int error_errno[FMA_MAX_ERROR_DEPTH];
};

/*
 * function prototypes
 */
void fma_perror(void);
void fma_save_error(char *, int, int, char *);
void fma_reset_error(void);
void fma_exit(int);
void fma_perror_exit(int);
void fma_init_standalone_vars(void);
void fma_init_standalone(void);
int fma_init_myri_vars(void);
void fma_init_tunnel_vars(void);
int fma_init_myri(void);
int fma_send_myri_info(int);
void fma_init_settings_vars(void);


/*
 * external definitions
 */

extern int errno;
extern struct fma_vars A;

/*
 * inline functions
 */
static inline void
fma_reset_errors()
{
  A.error_index = 0;
}

/*
 * some constants
 */

#define FMA_IFC_ROUTE_LEN 8	/* longest nic-nic route */
