#include "libfma.h"
#include "lf_fabric.h"
#include "lf_topo_map.h"

#include "fma.h"
#include "fma_fabric.h"

struct lf_xbar *
fma_create_xbar(
  int num_ports,
  int xbar_id,
  int quadrant_disable)
{
  struct lf_xbar *xp;

  xp = lf_alloc_xbar(num_ports);
  if (xp == NULL) FMA_ERROR("Allocating xbar");

  LF_CALLOC(xp->user, struct fma_xbar, 1);
  FMA_XBAR(xp)->parent = xp;
  LF_CALLOC(FMA_XBAR(xp)->link_down, int, xp->num_ports);
  LF_CALLOC(FMA_XBAR(xp)->link_load, int, xp->num_ports);

  xp->xbar_id = xbar_id;
  xp->quadrant_disable = quadrant_disable;

  return xp;

 except:
  return NULL;
}

void
fma_free_xbar(
  struct lf_xbar *xp)
{
  LF_FREE(FMA_XBAR(xp)->link_down);
  LF_FREE(FMA_XBAR(xp)->link_load);
  LF_FREE(FMA_XBAR(xp)->mf_info);
  LF_FREE(xp->user);
  lf_free_xbar(xp);
}

struct lf_nic *
fma_create_nic(
  int num_ports,
  lf_mac_addr_t mac_addr)
{
  struct lf_nic *nicp;

  nicp = lf_alloc_nic(num_ports);
  if (nicp == NULL) FMA_ERROR("Allocating nic");

  LF_MAC_COPY(nicp->mac_addr, mac_addr);

  LF_CALLOC(nicp->user, struct fma_nic, 1);
  FMA_NIC(nicp)->parent = nicp;
  LF_CALLOC(FMA_NIC(nicp)->link_down, int, nicp->num_ports);

  return nicp;

 except:
  return NULL;
}

void
fma_free_nic(
  struct lf_nic *nicp)
{
  /* free all the user stuff */
  LF_FREE(FMA_NIC(nicp)->link_down);
  LF_FREE(FMA_NIC(nicp)->mf_info);
  LF_FREE(FMA_NIC(nicp)->route_buf);
  LF_FREE(FMA_NIC(nicp)->route_len);
  LF_FREE(nicp->user);

  lf_free_nic(nicp);
}

struct lf_host *
fma_create_host()
{
  struct lf_host *hp;

  LF_CALLOC(hp, struct lf_host, 1);
  LF_CALLOC(hp->user, struct fma_host, 1);
  return hp;

 except:
  fma_perror_exit(1);
  return NULL;
}

void
fma_free_host(
  struct lf_host *hp)
{
  int n;

  for (n=0; n<hp->num_nics; ++n) {
    fma_free_nic(hp->nics[n]);
  }
  lf_free_host(hp);
}
