#ifndef _lf_myri_packet_h_
#define _lf_myri_packet_h_
/*
 * FMA Myrinet packets
 */

#include "lf_xbar32.h"

#define FMA_PACKET_TYPE (0x4000 + A.standalone)
#define LF_MAX_MYRI_RAW_PKT_LEN (1024 - MAX_FMA_ROUTE_LEN)

enum fma_packet_subtype {
  FMA_SUBTYPE_NIC_SCOUT,
  FMA_SUBTYPE_NIC_SCOUT_REPLY,
  FMA_SUBTYPE_XBAR_SCOUT,
  FMA_SUBTYPE_XBAR32_SCOUT,
  FMA_SUBTYPE_FMA_TUNNEL_START,
  FMA_SUBTYPE_FMA_TUNNEL_BODY,
  FMA_SUBTYPE_FMA_TUNNEL_ACK,
  FMA_SUBTYPE_FMS_ID_QUERY,
  FMA_SUBTYPE_FMS_ID_REPLY,
  FMA_SUBTYPE_XBAR_COMPARE,
  FMA_SUBTYPE_NO_OP,
  FMA_SUBTYPE_COUNT
};

/*
 * Reasons a NIC scout might be sent
 */
enum lf_nic_scout_reason {
  LF_NSR_NEIGHBOR_SCOUT,
  LF_NSR_VERIFY,
  LF_NSR_FMS_MAP,
  LF_NSR_STANDALONE_MAP
};
typedef enum lf_nic_scout_reason lf_nic_scout_reason_t;

#define FMA_PROTOCOL_ID 1

struct lf_myri_packet {
  uint16_t type_16;		/* mapper packet or new FMA packet type? */
  uint16_t subtype_16;		/* FMA subtype */

  /* other type-independent fields are after the union for x32 HW reasons */

  union {

    struct fma_nic_scout {	/* subtype = NIC_SCOUT */
      uint8_t reason_8;		/* enum lf_nic_scout_reason */
      uint8_t level_8;		/* used in mapper election */
      uint8_t max_mac_addr[6];	/* highest ordinal mac addr on sender */
      
      uint8_t reply_route_len_8;
      uint8_t reply_route[MAX_FMA_ROUTE_LEN];
    } nic_scout;

    struct fma_nic_scout_reply {	/* subtype = NIC_SCOUT_REPLY */
      uint32_t fma_protocol_32;	/* FMA protocol supported */
      uint8_t mac_addr[6];	/* MAC address of packet sender */
      uint8_t port_8;		/* port scout was received on */
      uint8_t num_ports_8;	/* number of ports on NIC */
      uint8_t firmware_type_8;	/* GM, MX, XM, etc. */
      uint8_t level_8;		/* used in mapper election */
      uint8_t max_mac_addr[6];	/* highest ordinal mac addr on sender */
    } nic_scout_reply;

    struct fma_xbar_scout {	/* subtype = FMA_SUBTYPE_XBAR_SCOUT */
      uint32_t pad;			/* make structure non-empty */
    } xbar_scout;

    struct fma_x3bar2_scout {	/* subtype = FMA_SUBTYPE_XBAR32_SCOUT */
      struct lf_xbar32_insert x32_insert;	/* zero before sending */
    } xbar32_scout;

    struct fma_query {	/* subtype = FMA_QUERY */
      uint8_t reply_route_len;
      uint8_t reply_route[MAX_FMA_ROUTE_LEN];
    } query;

    struct fma_query_reply {	/* subtype = FMA_QUERY_REPLY */
      uint16_t num_items;	/* number of items in the reply */
      uint8_t num_reply_packets; /* total number of reply packets */
      uint8_t reply_packets_seq; /* index of this reply packet */
      uint8_t data[1];		/* data of reply, see below for format */
    } query_reply;

    
    struct fma_set_route {  /* subtype = FMA_SET_ROUTE */
      uint8_t mac_addr[6];	/* MAC address of destination */
      uint8_t destport;		/* port on destination */
      uint8_t srcport;		/* source port for receiving NIC */
      uint8_t route_len;
      uint8_t route[MAX_FMA_ROUTE_LEN];

      uint8_t reply_route_len;
      uint8_t reply_route[MAX_FMA_ROUTE_LEN];
    } set_route;
    
    struct fma_set_route_reply {  /* subtype = FMA_SET_ROUTE_REPLY */
      uint8_t return_code;	/* 0 for success, non-zero otherwise,
			           firmware specific interpretation */
    } set_route_reply;


    struct fma_pause_sending {	/* subtype = FMA_PAUSE_SENDING */
      uint8_t route_len;
      uint8_t route[MAX_FMA_ROUTE_LEN];
    } pause_sending;
      
    struct fma_pause_sending_reply { /* subtype = FMA_PAUSE_SENDING_REPLY */
      uint32_t pad;			/* make structure non-empty */
    } pause_sending_reply;

    struct fma_resume_sending {	/* subtype = FMA_RESUME_SENDING */
      uint8_t route_len;
      uint8_t route[MAX_FMA_ROUTE_LEN];
    } resume_sending;
      
    struct fma_resume_sending_reply { /* subtype = FMA_RESUME_SENDING_REPLY */
      uint32_t pad;			/* make structure non-empty */
    } resume_sending_reply;

    struct fma_xbar_compare {		/* subtype = FMA_XBAR_COMPARE */
      uint32_t map_session_id_32;
      uint32_t unres_node_id_32;
      uint32_t unres_port_32;
      uint32_t test_node_id_32;
      uint32_t test_port_32;
    } xbar_compare;

    /* forward "data" out onto the wire */
    struct fma_forward {	/* subtype = FMA_FORWARD */
      uint8_t port;		/* local port to use */
      uint16_t data_len;
      uint8_t data[1];		
    } forward;

    /* structure defining the start of a "tunneled" message
     * (a larger, higher-level message that needs reliable delivery)
     */
    struct fma_tunnel_start {
      uint32_t msg_length_32;
      uint32_t msg_type_32;
      uint32_t msg_id_32;		/* Unique ID for entire message */
      uint8_t origin_mac_addr[6];	/* MAC address of query originator */
      uint8_t reply_route[MAX_FMA_ROUTE_LEN];
      uint8_t reply_route_len_8;
    } tunnel_start;
#define LF_MAX_TUNNEL_START_LEN (LF_MAX_MYRI_RAW_PKT_LEN -	\
	(2 * sizeof(uint16_t)					\
	 + sizeof(struct fma_tunnel_start) - 1))
#define LF_TUNNEL_START_PKT_LEN(L) (2 * sizeof(uint16_t)	\
             + sizeof(struct fma_tunnel_start) - 1 + (L))

    /* a fragment of a tunneled message */
    struct fma_tunnel_body {
      uint32_t msg_id_32;		/* message ID */
      uint32_t offset_32;		/* offset into message */
      uint32_t frag_len_32;		/* length of this fragment */
      uint8_t origin_mac_addr[6];	/* MAC address of query originator */
      uint8_t data[1];			/* data of the message */
    } tunnel_body;
#define LF_MAX_TUNNEL_FRAG_LEN (LF_MAX_MYRI_RAW_PKT_LEN -	\
	(2 * sizeof(uint16_t)					\
	 + sizeof(struct fma_tunnel_body) - 1))
#define LF_TUNNEL_BODY_PKT_LEN(L) (2 * sizeof(uint16_t)	\
             + sizeof(struct fma_tunnel_body) - 1 + (L))

    /* A message acknowledging receipt of a tunnel body fragment
     * This implicitly acknowledges all messages up through this offset
     */
    struct fma_tunnel_ack {
      uint32_t msg_id_32;		/* message ID */
      uint32_t offset_32;		/* offset into message */
    } tunnel_ack;
    
  } u;

  /*
   * The following fields are USUALLY present.  They are not present in:
   *    fma_tunnel_start
   *    fma_tunnel_body
   *    fma_tunnel_ack
   */

  /* the following three fields are returned unchanged by the responder */
  uint32_t serial_32;		/* request/reply serial number */
  
  uint8_t origin_mac_addr[6];	/* MAC address of query originator */
  uint8_t origin_ifc_8;		/* interface query originated from */
  uint8_t pad1;
};

/*
 * FMS-FMA tunnel packets are not part of the union above to save bytes since
 * we need to put as much as possible into the payload.
 */
struct lf_fms_fma_tunnel_hdr {
  uint16_t type_16;		/* FMA packet type */
  uint16_t subtype_16;		/* FMA subtype */

  uint32_t serial_no_32;	/* packet serial number */
  uint8_t proxy_mac_addr[6];	/* MAC address of proxy */

  uint8_t data[2];		/* data pad */
};


/* 
 * Format of FMA_QUERY_REPLY packet.
 * The data of an FMA_QUERY_REPLY is formatted in triplets consisting of
 * a type, a length, and arbitrary data associated with the type.  
 * An FMA_QUERY reply may generate multiple FMA_QUERY_REPLY packets due
 * to MTU limitations, so the reply structure allows for multi-packet
 * replies.
 *
 * The header of the QUERY_REPLY specifies the number of triplets in this
 * packet (num_items), also the number of reply packets (num_reply_packets),
 * and the number of this reply packet in:
 *  (0 <= reply_packet_seq < num_reply_packets)
 *
 * Each triplet has the format below:
 */
struct fma_triplet {
  uint16_t length;	/* total length in bytes including lenght&data_id */
  uint16_t data_id;	/* data ID of this triplet, data format is implicit */
  uint8_t data[1];	/* the data associated with the ID */
};

enum fma_query_data_id {
  FMA_QUERY_REPLY_NIC_TYPE,
  FMA_QUERY_REPLY_NIC_SERIAL_NUMBER,
  FMA_QUERY_REPLY_FIRMWARE_SPECIFIC_DATA,
  FMA_QUERY_REPLY_
};

/*
 * Convenience structures for triplets
 */
struct fma_triplet_nic_type {
  uint16_t length;		/* FMA_NIC_TYPE_LEN + 4 */
  uint16_t data_id;		/* FMA_QUERY_REPLY_NIC_TYPE */
  uint8_t nic_type[LF_SHORT_STRING_LEN];  /* null-terminated string */
};

struct fma_triplet_nic_serial {
  uint16_t length;		/* FMA_NIC_SERIAL_LEN + 4 */
  uint16_t data_id;		/* FMA_QUERY_REPLY_NIC_SERIAL_NUMBER */
  uint8_t nic_type[LF_SHORT_STRING_LEN];  /* null-terminated string */
};

struct fma_firmware_specific_data {
  uint16_t length;
  uint16_t data_id;		/* FMA_QUERY_REPLY_FIRMWARE_SPECIFIC_DATA */
  uint8_t data[1];
};

#endif /* _lf_myri_packet_h_ */
